/* This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

Copyright (C) 2000 Liam Girdwood <liam@novalinux.com>
*/

/*
** Thanks to Jean Meeus for most of the algorithms used in this library.
** From his book "Astronomical Algorithms".
*/

#ifndef LN_LIBNOVA_H
#define LN_LIBNOVA_H

#define LN_HIGH_ACCURACY 1
#define LN_LOW_ACCURACY 0

#ifdef __cplusplus
extern "C" {
#endif

/*
** Date
*/

struct ln_date
{
    int years;
    int months;
    int days;
    int hours;
    int minutes;
    double seconds;
};

/*
** Angle in degrees, minutes and seconds
*/

struct ln_dms
{
    int degrees;
    int minutes;
    double seconds;
};

/*
** Angle in hours, minutes and seconds
*/

struct ln_hms
{
    int hours;
    int minutes;
    double seconds;
};

/*
** Equatorial Coordinates.
*/

struct ln_equ_position
{
    struct ln_hms ra;
    struct ln_dms dec;
};

/*
** Horizontal Coordinates.
*/

struct ln_horiz_position
{
    struct ln_dms az;
    struct ln_dms alt;
};

/*
** Longitude and Latitude.
*/

struct ln_long_lat_position
{
    struct ln_dms lng;
    struct ln_dms lat;
};

/*
** Coordinates, Horizontal or Equatorial. Use this to pass positional data between
** functions rather than ln_equ_position or ln_horiz_position
** */

union ln_position
{
    struct ln_equ_position equ;
    struct ln_horiz_position hrz;
    struct ln_long_lat_position ll;
};

/* solar coordinates */
struct ln_solar_position
{
	union ln_position geo;
	double radius_vector;
};

/* heliocentric position */
struct ln_heliocentric_position
{
	double L;
	double B;
	double R;
};

/*
**   Library Version Number
*/

char * get_ln_version (void);

/*
** Library Accuracy
*/

void set_ln_accuracy (int accuracy);
extern int ln_accuracy;

/*
**    General Calendar Functions 
*/
 
/* get the Julian day from date and vice versa*/
double get_julian_day (struct ln_date * date);
struct ln_date * get_date (double JD);

/* get day of the week from date (0 = Sunday .. 6 = Saturday) */
unsigned int get_day_of_week (struct ln_date *date);


/*
[]------------------------------------------------------------------------[]
|                                                                          |
|  get_dec_location()                                                      |
|  Obtains Latitude, Longitude, RA or Declination from a string.           |
|  If the last char is N/S doesn't accept more than 90 degrees.            |
|  If it is E/W doesn't accept more than 180 degrees.                      |
|  If they are hours don't accept more than 24:00                          |
|                                                                          |
|  Any position can be expressed as follows:                               |
|  (please use a 8 bits charset if you want                                |
|  to view the degrees separator char '0xba')                              |
|                                                                          |
|  42.30.35,53                                                             |
|  900'0,01 W                                                             |
|  4230'35.53 N                                                           |
|  4230'35.53S                                                            |
|  4230'N                                                                 |
|  - 42.30.35.53                                                           |
|   42:30:35.53 S                                                          |
|  + 42.30.35.53                                                           |
|  +4230 35,53                                                            |
|   23h36'45,0                                                             |
|                                                                          |
|                                                                          |
|  42:30:35.53 S = -4230'35.53"                                           |
|  + 42 30.35.53 S the same previous position, the plus (+) sign is        |
|  considered like an error, the last 'S' has precedence over the sign     |
|                                                                          |
|  900'0,01 N ERROR: +- 900'00.00" latitude limit                        |
|                                                                          |
[]------------------------------------------------------------------------[]
*/
double get_dec_location(char *s);



/*
[]------------------------------------------------------------------------[]
|                                                                          |
|  get_humanr_location()                                                   |
|  obtains a human readable location in the form: ddmm'ss.ss"             |
|                                                                          |
[]------------------------------------------------------------------------[]
*/

char *get_humanr_location(double location);


/*
**    Dynamical Time 
*/

/* get approximate Dynamical time difference from JD in seconds */
double get_dynamical_time_diff (double JD);

/* get julian ephemeris day */
double get_jde (double JD);

/*
**     Sidereal Time 
*/

/* get sidereal time from date  */
/* returns sidereal time in hours */
double get_mean_sidereal_time (double JD); 
double get_apparent_sidereal_time (double JD);

/*
**    General Conversion Functions 
*/

/* radians to degrees and vice versa */
double rad_to_deg (double radians);
double deg_to_rad (double degrees);

/* hours to degrees and vice versa */
double hms_to_deg (struct ln_hms * hms);
struct ln_hms * deg_to_hms_ret (double degrees);
void deg_to_hms (double degrees, struct ln_hms * hms);


/* hours to radians and vice versa */
double hms_to_rad (struct ln_hms * hms);
struct ln_hms * rad_to_hms_ret (double radians);
void rad_to_hms(double radians, struct ln_hms * hms);

/* dms to degrees and vice versa */
struct ln_dms * deg_to_dms_ret (double degrees);
double dms_to_deg (struct ln_dms *dms);
void deg_to_dms (double degrees, struct ln_dms * dms);

/* dms to radians and vice cersa */
struct ln_dms * rad_to_dms_ret (double radians);
double dms_to_rad (struct ln_dms *dms);
void rad_to_dms (double radians, struct ln_dms * dms);


/* add seconds to hms */
void add_secs_hms (struct ln_hms * hms, double seconds);

/* add hms to hms */
void add_hms (struct ln_hms * source, struct ln_hms * dest);

/* puts a large angle in the correct range 0 - 360 degrees */
double range_degrees (double angle);

/* puts a large angle in the correct range 0 - 2PI radians */
double range_radians (double angle);


/*
**    Transformation of Coordinates 
**    using apparent positions 
*/

/* get horizontal coordinates from equatorial coordinates */
/* Equ 12.5,12.6 pg 88 */

struct ln_horiz_position * get_horizontal_from_equatorial (struct ln_equ_position * object, struct ln_long_lat_position * observer, double JD);

/* get equatorial coordinates from ecliptical coordinates */
/* Equ 12.3, 12.4 pg 89 */
struct ln_equ_position * get_equatorial_from_ecliptical (struct ln_long_lat_position * object, double JD);

/* get ecliptical cordinates from equatorial coordinates */
/* Equ 12.1, 12.2 Pg 88 */
struct ln_long_lat_position * get_ecliptical_from_equatorial (struct ln_equ_position * object, double JD);

/* get equatorial coordinates from horizontal coordinates  */
/* Pg 89 */
struct ln_equ_position * get_equatorial_from_horizontal (struct ln_horiz_position *object, struct ln_long_lat_position * observer, double JD); 

/*
** VSOP87
*/

/* transform from VSOP87 to FK5 */
/* equation 31.3 Pg 207         */
/* t is jullian millennia from 2000 */
void vsop87_to_fk5 (struct ln_heliocentric_position * position, double t);


/*
**    Earths Globe 
*/

/* get Earth globe centre distance */
void get_earth_centre_distance (float height, double latitude, double * p_sin_angle, double * p_cos_angle);

/*
**    Nutation
*/

/* get nutation in degrees for longitude, obliquity and ecliptic */
void get_nutation (double JD, double * longitude, double * obliquity, double * ecliptic);




/*
** Apparent place of an Object
*/

/* get the apparent position of an object from its mean position */
/* uses mean equatorial coordinates */
struct ln_equ_position * get_apparent_position (struct ln_equ_position * mean_position, struct ln_equ_position * proper_motion, double JD);


/*
** Precession.
*/

/* get equatorial/ecliptical coordinates with the effects of precession for a given Julian Day */

/* uses mean equatorial coordinates*/
/* Equ 20.2, 20.3, 20.4 pg 126 */
struct ln_equ_position * get_precession_equatorial (struct ln_equ_position * mean_position, double JD);

/* uses mean ecliptical coordinates*/
/* Equ 20.5, 20.6 pg 128 */
struct ln_long_lat_position * get_precession_ecliptical (struct ln_long_lat_position * mean_position, double JD); 


/*
** Proper Motion.
*/

/* get equatorial coordinates with the effects of proper motion for a given Julian Day */

/* uses mean equatorial coordinates*/
/* Equ 20.2, 20.3, 20.4 pg 126 */
struct ln_equ_position * get_proper_motion_equatorial (struct ln_equ_position * mean_position, struct ln_equ_position * proper_motion, double JD);


/*
** Aberration
*/

/* get equatorial/ecliptical coordinates with the effects of Aberration and nutation for a given Julian Day */
/* has low and high accuracy methods for equatorial coordinates*/
/* uses mean equatorial coordinates*/
/* Equ 22.1, 22.3, 22.4 an Ron-Vondrak expression */
struct ln_equ_position * get_aberration_equatorial (struct ln_equ_position * mean_position, double JD);

/* uses mean ecliptical coordinates*/
/* Equ 22.1, 22.2 pg 139 */
struct ln_long_lat_position * get_aberration_ecliptical (struct ln_long_lat_position * mean_position, double JD); 


/*
** Solar Coordinates
*/

/* get the solar ecliptical/equatorial coordinates for a given julian date */

/* low accuracy mode 0.0006 degree - 0.0009 degree error */
/* high accuracy mode 0.01 arc second error - uses VSOP87 */

/* geometric longitude, latitude and radius vector */
struct ln_solar_position * get_geometric_solar_coordinates (double JD);

/* apparent equatorial coordinates */
struct ln_equ_position * get_solar_coordinates_equatorial (double JD);

/* apparent ecliptical coordinates */
struct ln_long_lat_position * get_solar_coordinates_ecliptical (double JD);


/*
** Mercury
*/

/* get Mercury heliocentric coordinates */
/* Chapter 31 Pg 206-207 Equ 31.1 31.2 , 31.3 using VSOP 87 */
struct ln_heliocentric_position * get_mercury_heliocentric_coordinates (double JD);


/*
** Venus
*/

/* get venus heliocentric coordinates */
/* Chapter 31 Pg 206-207 Equ 31.1 31.2 , 31.3 using VSOP 87 */
struct ln_heliocentric_position * get_venus_heliocentric_coordinates (double JD);


/*
** Earth
*/

/* get earth heliocentric coordinates */
/* Chapter 31 Pg 206-207 Equ 31.1 31.2 , 31.3 using VSOP 87 */
struct ln_heliocentric_position * get_earth_heliocentric_coordinates (double JD);


/*
** Mars
*/

/* get mars heliocentric coordinates */
/* Chapter 31 Pg 206-207 Equ 31.1 31.2 , 31.3 using VSOP 87 */
struct ln_heliocentric_position * get_mars_heliocentric_coordinates (double JD);


/*
** Jupiter
*/

/* get jupiter heliocentric coordinates */
/* Chapter 31 Pg 206-207 Equ 31.1 31.2 , 31.3 using VSOP 87 */
struct ln_heliocentric_position * get_jupiter_heliocentric_coordinates (double JD);


/*
** Saturn
*/

/* get saturn heliocentric coordinates */
/* Chapter 31 Pg 206-207 Equ 31.1 31.2 , 31.3 using VSOP 87 */
struct ln_heliocentric_position * get_saturn_heliocentric_coordinates (double JD);


/*
** Uranus
*/

/* get Uranus heliocentric coordinates */
/* Chapter 31 Pg 206-207 Equ 31.1 31.2 , 31.3 using VSOP 87 */
struct ln_heliocentric_position * get_uranus_heliocentric_coordinates (double JD);


/*
** Neptune
*/

/* get neptune heliocentric coordinates */
/* Chapter 31 Pg 206-207 Equ 31.1 31.2 , 31.3 using VSOP 87 */
struct ln_heliocentric_position * get_neptune_heliocentric_coordinates (double JD);




#ifdef __cplusplus
};
#endif


#endif
